using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;

namespace Jeffz.Web
{
	[PersistChildren(false)]
	[ParseChildren(true)]
	[NonVisualControl]
	public class SwitchPartManager : Control
	{
		private const string HiddenElementName = "__PartType__";

		private bool initialized = false;

		private string partTypeToSave = null;

		public static SwitchPartManager GetCurrent(Page page)
		{
			return page.Items[typeof(SwitchPartManager)] as SwitchPartManager;
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if (this.Page.Items.Contains(typeof(SwitchPartManager)))
			{
				throw new InvalidOperationException("One SwitchPartManager per page.");
			}

			this.Page.Items[typeof(SwitchPartManager)] = this;

			this.Page.InitComplete += new EventHandler(Page_InitComplete);
			this.Page.PreRenderComplete += new EventHandler(Page_PreRenderComplete);
		}

		private void Page_InitComplete(object sender, EventArgs e)
		{
			this.initialized = true;

			string partType = this.Page.Request.Params[SwitchPartManager.HiddenElementName];
			if (partType != null)
			{
				this.SwitchTo(partType);
			}
		}

		private void Page_PreRenderComplete(object sender, EventArgs e)
		{
			if (!String.IsNullOrEmpty(this.partTypeToSave))
			{
				ScriptManager.RegisterHiddenField(this.Page, SwitchPartManager.HiddenElementName, this.partTypeToSave);
			}
		}

		private string _PlaceHolderUpdatePanelID;

		[IDReferenceProperty(typeof(UpdatePanel))]
		public string PlaceHolderUpdatePanelID
		{
			get
			{
				return this._PlaceHolderUpdatePanelID;
			}
			set
			{
				if (this.initialized)
				{
					throw new InvalidOperationException("Please set the PlaceHolderUpdatePanelID property declaratively.");
				}

				this._PlaceHolderUpdatePanelID = value;
			}
		}

		private UpdatePanel _PlaceHolderUpdatePanel;

		private UpdatePanel PlaceHolderUpdatePanel
		{
			get
			{
				if (this._PlaceHolderUpdatePanel == null)
				{
					if (this.PlaceHolderUpdatePanelID == null)
					{
						throw new InvalidOperationException("Please specify the placeholder.");
					}

					this._PlaceHolderUpdatePanel = this.NamingContainer.FindControl(this.PlaceHolderUpdatePanelID) as UpdatePanel;

					if (this._PlaceHolderUpdatePanel == null)
					{
						throw new InvalidOperationException("Please specify the valid PlaceHolderUpdatePanelID property.");
					}
				}

				return this._PlaceHolderUpdatePanel;
			}
		}

		public void SwitchTo(string partType)
		{
			Control container = this.PlaceHolderUpdatePanel.ContentTemplateContainer;
			container.Controls.Clear();

			Control control = this.Page.LoadControl(partType + ".ascx");
			control.ID = "JustToPreserveUniqueName";
			container.Controls.Add(control);

			this.partTypeToSave = partType;
		}
	}
}
